//-----------------------------------------------------------------------------
// Desc: Example code showing how to do billboarding. The sample uses
//       billboarding to draw some trees.
//
//       Note: This implementation is for billboards that are fixed to rotate
//       about the Y-axis, which is good for things like trees. For
//       unconstrained billboards, like explosions in a flight sim, the
//       technique is the same, but the the billboards are positioned slightly
//       differently. Try using the inverse of the view matrix, TL-vertices, or
//       some other technique.
//
// Copyright (c) 1995-2001 Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
unit Unit1;

interface
uses
  Windows, Messages, Classes, Forms, Menus, OmegaScreen,
  OmegaBitmapFont, OmegaInput, OmegaTimer, D3DX8, D3DFont,
  D3DFile, D3DApp, D3DUtil, DXUtil, OmegaUtils,
  Direct3D8;

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    m_fTime: single;
    function FrameMove: HRESULT;
    function InitData: HRESULT;
    function ResetData: HRESULT;
   public
    function DrawTrees: HRESULT;     
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

const
  NUMTREETEXTURES = 3;
  NUM_TREES = 500;

// Custom vertex type for the trees
type
  PTreeVertex = ^TTreeVertex;
  TTreeVertex = packed record
    p: TD3DXVector3;    // Vertex position
    color: DWORD;       // Vertex color
    tu, tv: Single;     // Vertex texture coordinates
  end;
  PATreeVertex = ^ATreeVertex;
  ATreeVertex = array[0..0] of TTreeVertex;

//-----------------------------------------------------------------------------
// Name: Tree
// Desc: Simple structure to hold data for rendering a tree
//-----------------------------------------------------------------------------
type
  PTree = ^Tree;
  Tree = record
    v: array[0..3] of TTreeVertex; // Four corners of billboard quad
    vPos: TD3DXVector3;            // Origin of tree
    dwTreeTexture: DWORD;          // Which texture map to use
    dwOffset: DWORD;               // Offset into vertex buffer of tree's vertices
  end;

var
    m_pTerrain: CD3DMesh;           // Terrain object
    m_pSkyBox: CD3DMesh;            // Skybox background object
    m_pTreeVB: IDirect3DVertexBuffer8;  // Vertex buffer for rendering a tree
    m_pTreeTextures: array[0..NUMTREETEXTURES-1] of IDirect3DTexture8; // Tree images
    m_matBillboardMatrix: TD3DXMatrix; // Used for billboard orientation
    m_Trees: array[0..NUM_TREES-1] of Tree;   // Array of tree info

const
  D3DFVF_TREEVERTEX = (D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1);

  // Tree textures to use
  g_strTreeTextures: array[0..2] of PChar = (
    'Tree02S.tga',
    'Tree35S.tga',
    'Tree01S.tga');

// Need global access to the eye direction used by the callback to sort trees
var
  g_vDir: TD3DXVector3;

// Simple function to define "hilliness" for terrain
function HeightField(x,y: Single): Single;
begin
  Result:= 9*(cos(x/20+0.2)*Cos(y/15-0.2)+1.0);
end;

function TForm1.ResetData: HRESULT;
var
  matProj: TD3DXMatrix;
  fAspect: Single;
begin
  // Set the transform matrices (view and world are updated per frame)
  fAspect := ClientWidth / ClientHeight;
  D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 1.0, 100.0);
  with OmegaScreen1.Device do
  begin
   SetTransform(D3DTS_PROJECTION, matProj);
   // Set up the default texture states
   SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
   SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
   SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
   SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
   SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_ADDRESSU,  D3DTADDRESS_CLAMP);
   SetTextureStageState(0, D3DTSS_ADDRESSV,  D3DTADDRESS_CLAMP);
   SetRenderState(D3DRS_DITHERENABLE, iTrue);
   SetRenderState(D3DRS_ZENABLE,      iTrue);
   SetRenderState(D3DRS_LIGHTING,     iFalse);
   m_pTerrain.RestoreDeviceObjects(OmegaScreen1.Device);
   m_pSkyBox.RestoreDeviceObjects(OmegaScreen1.Device);
  end;
  Result:= S_OK;
end;

function TForm1.InitData: HRESULT;
type
  TVertex = packed record
    x, y, z, tu, tv: Single;
  end;
  PAVertex = ^AVertex;
  AVertex = array[0..0] of TVertex;
var
  i: DWORD;
  v: PATreeVertex;
  iTree: Integer;
  dwOffset: DWORD;
  pVB: IDirect3DVertexBuffer8;
  pVertices: PAVertex;
  dwNumVertices: DWORD;
begin
  // Create the tree textures
  for i:= 0 to NUMTREETEXTURES - 1 do
  begin
    if FAILED(D3DUtil_CreateTexture(OmegaScreen1.Device, g_strTreeTextures[i], m_pTreeTextures[i]))
    then Exit;
  end;
  // Create a quad for rendering each tree
  if FAILED(OmegaScreen1.Device.CreateVertexBuffer(NUM_TREES*4*SizeOf(TTreeVertex),
                                            D3DUSAGE_WRITEONLY, D3DFVF_TREEVERTEX,
                                            D3DPOOL_MANAGED, m_pTreeVB)) then
  begin
    Result:= E_FAIL;
    Exit;
  end;
  // Copy tree mesh data into vertexbuffer
  m_pTreeVB.Lock(0, 0, PByte(v), 0);
  dwOffset := 0;
  for iTree := 0 to NUM_TREES - 1 do
  begin
    Move(m_Trees[iTree].v, v[dwOffset], 4*SizeOf(TTreeVertex));
    m_Trees[iTree].dwOffset := dwOffset;
    dwOffset := dwOffset+4;
  end;
  m_pTreeVB.Unlock;
  // Load the skybox
  if FAILED(m_pSkyBox.Create_(OmegaScreen1.Device, PChar('SkyBox2.x')))
   then Exit;
  // Load the terrain
  if FAILED(m_pTerrain.Create_(OmegaScreen1.Device, PChar('SeaFloor.x')))
   then Exit;
  // Add some "hilliness" to the terrain
  if SUCCEEDED(m_pTerrain.GetSysMemMesh.GetVertexBuffer(pVB)) then
  begin
    dwNumVertices := m_pTerrain.GetSysMemMesh.GetNumVertices;
    pVB.Lock(0, 0, PBYTE(pVertices), 0);
    for i:= 0 to dwNumVertices - 1 do
      pVertices[i].y := HeightField(pVertices[i].x, pVertices[i].z);
    pVB.Unlock;
    pVB := nil;
  end;
  Result:= S_OK;
end;

procedure TForm1.FormCreate(Sender: TObject);
const
  RAND_MAX = $7fff;
var
  i: DWORD;
  fTheta, fRadius: Single;
  fWidth, fHeight: Single;
  r, g, b, dwcolor: DWORD;
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 m_pSkyBox      := CD3DMesh.Create(PChar('D3D Mesh'));
 m_pTerrain     := CD3DMesh.Create(PChar('D3D Mesh'));
 m_pTreeVB      := nil;
 for i := 0 to NUMTREETEXTURES - 1 do
  m_pTreeTextures[i] := nil;
  Randomize;
  // Initialize the tree data
  for i:= 0 to NUM_TREES-1 do
  begin
    // Position the trees randomly
    fTheta  := (2.0*D3DX_PI*random(RAND_MAX))/RAND_MAX;
    fRadius := (25.0 + 55.0 * random(RAND_MAX))/RAND_MAX;
    m_Trees[i].vPos.x := fRadius * Sin(fTheta);
    m_Trees[i].vPos.z := fRadius * Cos(fTheta);
    m_Trees[i].vPos.y := HeightField(m_Trees[i].vPos.x, m_Trees[i].vPos.z);
    // Size the trees randomly
    fWidth  := 1.0 + 0.2 * (random(RAND_MAX)-random(RAND_MAX))/RAND_MAX;
    fHeight := 1.4 + 0.4 * (random(RAND_MAX)-random(RAND_MAX))/RAND_MAX;
    // Each tree is a random color between red and green
    r := (255-190) + trunc((190*(random(RAND_MAX))/RAND_MAX));
    g := (255-190) + trunc((190*(random(RAND_MAX))/RAND_MAX));
    b := 0;
    dwColor := $ff000000 + (r shl 16) + (g shl 8) + (b shl 0);
    m_Trees[i].v[0].p     := D3DXVector3(-fWidth, 0*fHeight, 0.0);
    m_Trees[i].v[0].color := dwColor;
    m_Trees[i].v[0].tu    := 0.0;   m_Trees[i].v[0].tv := 1.0;
    m_Trees[i].v[1].p     := D3DXVector3(-fWidth, 2*fHeight, 0.0);
    m_Trees[i].v[1].color := dwColor;
    m_Trees[i].v[1].tu    := 0.0;   m_Trees[i].v[1].tv := 0.0;
    m_Trees[i].v[2].p     := D3DXVector3( fWidth, 0*fHeight, 0.0);
    m_Trees[i].v[2].color := dwColor;
    m_Trees[i].v[2].tu    := 1.0;   m_Trees[i].v[2].tv := 1.0;
    m_Trees[i].v[3].p     := D3DXVector3( fWidth, 2*fHeight, 0.0);
    m_Trees[i].v[3].color := dwColor;
    m_Trees[i].v[3].tu    := 1.0;   m_Trees[i].v[3].tv := 0.0;
    // Pick a random texture for the tree
    m_Trees[i].dwTreeTexture := Trunc((NUMTREETEXTURES * random(RAND_MAX)) / RAND_MAX);
  end;
 InitData;
 ResetData;
 DXUtil_Timer(TIMER_START);
 OmegaTimer1.Enabled := True;
end;

//-----------------------------------------------------------------------------
// Name: TreeSortCB()
// Desc: Callback function for sorting trees in back-to-front order
//-----------------------------------------------------------------------------
function TreeSortCB(const arg1, arg2: Pointer): Integer;
const
  Epsilon = 1e-5;
var
  p1,p2:PTree;
  d1,d2:Single;
begin
  p1 := arg1;
  p2 := arg2;
  d1 := p1.vPos.x * g_vDir.x + p1.vPos.z * g_vDir.z;
  d2 := p2.vPos.x * g_vDir.x + p2.vPos.z * g_vDir.z;
  if Abs(d1-d2) < Epsilon then Result:= 0
  else if (d1 < d2) then Result:= +1
  else Result:= -1;
end;

function TForm1.FrameMove: HRESULT;
var
  vUpVec: TD3DXVector3;
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  matView: TD3DXMatrix;
  vDir: TD3DXVector3;
begin
  m_fTime := DXUtil_Timer(TIMER_GETAPPTIME);

  // Get the eye and lookat points from the camera's path
  vUpVec := D3DXVector3(0.0, 1.0, 0.0);
  vEyePt.x := 30.0 * Cos(0.8*m_fTime);
  vEyePt.z := 30.0 * Sin(0.8*m_fTime);
  vEyePt.y := 4 + HeightField(vEyePt.x, vEyePt.z);
  vLookatPt.x := 30.0 * Cos(0.8 * (m_fTime + 0.5));
  vLookatPt.z := 30.0 * Sin(0.8 * (m_fTime + 0.5));
  vLookatPt.y := vEyePt.y - 1.0;
  // Set the app view matrix for normal viewing
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
  // Set up a rotation matrix to orient the billboard towards the camera.
  D3DXVec3Subtract(vDir, vLookatPt, vEyePt);
  // vDir := VectorSub(vLookatPt , vEyePt);
  if (vDir.x > 0.0) then
    D3DXMatrixRotationY(m_matBillboardMatrix, -arctan(vDir.z/vDir.x)+D3DX_PI/2)
  else
    D3DXMatrixRotationY(m_matBillboardMatrix, -arctan(vDir.z/vDir.x)-D3DX_PI/2);
  // Sort trees in back-to-front order
  g_vDir := vDir;
  QSort(@m_Trees, NUM_TREES, SizeOf(Tree), TreeSortCB);
  Result:= S_OK;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var
  matView, matViewSave: TD3DXMatrix;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
 begin
  OmegaScreen1.ToggleFullScreen;
  ResetData;
 end;
 FrameMove;
 OmegaScreen1.ClearScreen(255, 0, 0);
 OmegaScreen1.BeginRender;

 // Center view matrix for skybox and disable zbuffer
 OmegaScreen1.Device.GetTransform(D3DTS_VIEW, matViewSave);
 matView := matViewSave;
 matView._41 := 0.0; matView._42 := -0.3; matView._43 := 0.0;
 OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
 OmegaScreen1.Device.SetRenderState(D3DRS_ZENABLE, iFalse);
 // Some cards do not disable writing to Z when
 // D3DRS_ZENABLE is FALSE. So do it explicitly
 OmegaScreen1.Device.SetRenderState(D3DRS_ZWRITEENABLE, iFalse);
 // Render the skybox
 m_pSkyBox.Render(OmegaScreen1.Device);
 // Restore the render states
 OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matViewSave);
 OmegaScreen1.Device.SetRenderState(D3DRS_ZENABLE, iTrue);
 OmegaScreen1.Device.SetRenderState(D3DRS_ZWRITEENABLE, iTrue);
 // Draw the terrain
 m_pTerrain.Render(OmegaScreen1.Device);
 // Draw the trees
 DrawTrees;

// OmegaScreen1.SetBlendMode(bmNormal);
// OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

function TForm1.DrawTrees: HRESULT;
var
  i: DWORD;
  matWorld: TD3DXMatrix;
begin
 with OmegaScreen1.Device do
 begin
  // Set diffuse blending for alpha set in vertices.
  SetRenderState(D3DRS_ALPHABLENDENABLE, iTrue);
  SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  // Enable alpha testing (skips pixels with less than a certain alpha.)
  if (OmegaScreen1.FD3DDEVCAPS8.AlphaCmpCaps and D3DPCMPCAPS_GREATEREQUAL) = D3DPCMPCAPS_GREATEREQUAL then
  begin
    SetRenderState(D3DRS_ALPHATESTENABLE, iTrue);
    SetRenderState(D3DRS_ALPHAREF,        $08);
    SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
  end;
  // Loop through and render all trees
  SetStreamSource(0, m_pTreeVB, SizeOf(TTreeVertex));
  SetVertexShader(D3DFVF_TREEVERTEX);
  for i:= 0 to NUM_TREES - 1 do
  begin
    // Set the tree texture
    SetTexture(0, m_pTreeTextures[m_Trees[i].dwTreeTexture]);
    // Translate the billboard into place
    m_matBillboardMatrix._41 := m_Trees[i].vPos.x;
    m_matBillboardMatrix._42 := m_Trees[i].vPos.y;
    m_matBillboardMatrix._43 := m_Trees[i].vPos.z;
    SetTransform(D3DTS_WORLD, m_matBillboardMatrix);
    // Render the billboard
    DrawPrimitive(D3DPT_TRIANGLESTRIP, m_Trees[i].dwOffset, 2);
  end;
  // Restore state
  D3DXMatrixIdentity(matWorld);
  SetTransform(D3DTS_WORLD, matWorld );
  SetRenderState(D3DRS_ALPHATESTENABLE,  iFalse);
  SetRenderState(D3DRS_ALPHABLENDENABLE, iFalse);
 end;
 Result:= S_OK;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var i: DWORD;
begin
 m_pTerrain.Destroy_;
 m_pSkyBox.Destroy_;
 for i := 0 to NUMTREETEXTURES - 1 do
  m_pTreeTextures[i] := nil;
 m_pTreeVB := nil;
end;

end.





