unit OmegaThreadTimer;

interface

uses
  Classes;

type
  TOmegaThreadTimer = class;

  TOmegaThreadTimerThread = class(TThread)
  private
    FOmegaThreadTimer: TOmegaThreadTimer;
  public
    procedure Execute; override;
  end;

  TOmegaThreadTimer = class(TComponent)
  private
    FQPCAvailable: Boolean;
    FFrequency: Int64;
    FOldTime, FNewTime: Int64;
    FDelta: Double;
    FThread: TOmegaThreadTimerThread;
    FFPSOldTime: Int64;
    FFPS, FFPSCurrent: Integer;
    FPriority: TThreadPriority;
    FEnabled: Boolean;
    FTargetFPS: Integer;
    FTargetFPSDelta: Double;
    FOnTimer: TNotifyEvent;
    procedure SetPriority(const Value: TThreadPriority);
    procedure SetEnabled(const Value: Boolean);
    procedure SetTargetFPS(const Value: Integer);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Delta: Double read FDelta;
    property FPS: Integer read FFPS;
  published
    property Priority: TThreadPriority read FPriority write SetPriority;
    property Enabled: Boolean read FEnabled write SetEnabled;
    property TargetFPS: Integer read FTargetFPS write SetTargetFPS;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  end;

implementation

uses
  Windows, MMSystem;

{ TOmegaThreadTimerThread }

procedure TOmegaThreadTimerThread.Execute;
begin
  with FOmegaThreadTimer do begin
    while not Terminated do begin
      if FQPCAvailable then
        QueryPerformanceCounter(FNewTime)
      else
        FNewTime := timeGetTime;
      if FNewTime - FFPSOldTime >= FFrequency then begin
        FFPS := FFPSCurrent;
        FFPSCurrent := 0;
        FFPSOldTime := FNewTime;
      end;
      if FTargetFPS > 0 then begin
        if (FNewTime - FOldTime) >= FTargetFPSDelta then begin
          FDelta := (FNewTime - FOldTime) / FTargetFPSDelta;
          if Assigned(FOnTimer) then FOnTimer(FOmegaThreadTimer);
          FOldTime := FNewTime;
          Inc(FFPSCurrent);
        end;
      end else begin
        FDelta := (FNewTime - FOldTime) / FFrequency;
        if Assigned(FOnTimer) then FOnTimer(FOmegaThreadTimer);
        FOldTime := FNewTime;
        Inc(FFPSCurrent);
      end;
    end;
  end;
end;

{ TOmegaThreadTimer }

constructor TOmegaThreadTimer.Create(AOwner: TComponent);
begin
  inherited;

  FPriority := tpNormal;

  if csDesigning in ComponentState then Exit;

  //Set up available timing mechanisms on the host machine
  FQPCAvailable := QueryPerformanceFrequency(FFrequency);
  if not FQPCAvailable then begin
    timeBeginPeriod(1);
    FFrequency := 1000;
  end;

  FThread := TOmegaThreadTimerThread.Create(True);
  FThread.FreeOnTerminate := True;
  FThread.Priority := FPriority;
  FThread.FOmegaThreadTimer := Self;
end;

destructor TOmegaThreadTimer.Destroy;
begin
  if csDesigning in ComponentState then Exit;

  if FThread <> nil then
    FThread.Terminate; //Frees thread automatically!

  if not FQPCAvailable then
    timeEndPeriod(1);

  inherited;
end;

procedure TOmegaThreadTimer.SetEnabled(const Value: Boolean);
begin
  if Value = FEnabled then Exit;
  FEnabled := Value;

  if csDesigning in ComponentState then Exit;

  if Value then begin
    if FQPCAvailable then
      QueryPerformanceCounter(FOldTime)
    else
      FOldTime := timeGetTime;
    FFPSOldTime := FOldTime;
    FFPSCurrent := 0;
    FFPS := 0;
    FThread.Resume;
  end else
    FThread.Suspend;
end;

procedure TOmegaThreadTimer.SetPriority(const Value: TThreadPriority);
begin
  if Value = FPriority then Exit;
  FPriority := Value;
  if csDesigning in ComponentState then Exit;
  FThread.Priority := Value;
end;

procedure TOmegaThreadTimer.SetTargetFPS(const Value: Integer);
begin
  if Value = FTargetFPS then Exit;
  FTargetFPS := Value;

  if csDesigning in ComponentState then Exit;

  FTargetFPSDelta := FFrequency / FTargetFPS
end;

end.
